#include <VDream/Common/VCommon.h>
#include <VDream/Log/VLog.h>

// ----------------------------------------------------------------------------
// Param
// ----------------------------------------------------------------------------
class Param
{
public:
  QString   msg;
  bool      flush;
  bool      linefeed;
  VDuration interval;
  int       count;
  bool      set_stdin;
  bool      set_stdout;
  bool      set_stderr;

  Param()
  {
    msg        = "";
    flush      = false;
    linefeed   = true;
    interval   = 1000;
    count      = 5;
    set_stdin  = false;
    set_stdout = false;
    set_stderr = false;
  }

  bool parse(int argc, char* argv[])
  {
    int i = 1;
    while (i < argc)
    {
      QString s = argv[i];
      if (s == "-f")        flush = true;
      else if (s == "-nl")  linefeed = false;
      else if (s == "-i")   interval = QString(argv[i + 1]).toInt(), i++;
      else if (s == "-c")   count = QString(argv[i + 1]).toInt(), i++;
      else if (s == "-ssi") set_stdin = true;
      else if (s == "-sso") set_stdout = true;
      else if (s == "-sse") set_stderr = true;
      else if (msg == "")   msg = s;
      else
      {
        printf("invalid option %s\n", qPrintable(s));
        return false;
      }
      i++;
    }
    return true;
  }

  static void usage()
  {
    printf("src [msg] [option]\n");
    printf("-f            : flush stdout(default false)\n");
    printf("-nl           : no linefeed(default true)\n");
    printf("-i <interval> : interval(default 1000 msec)\n");
    printf("-c <count>    : count(default 5)\n");
    printf("-ssi          : setbuf stdin(default false)\n");
    printf("-sso          : setbuf stdout(default false)\n");
    printf("-sse          : setbuf stderr(default false)\n");
  }
};

// ----------------------------------------------------------------------------
// main
// ----------------------------------------------------------------------------
int main(int argc, char *argv[])
{
  Param param;
  if (!param.parse(argc, argv))
  {
    Param::usage();
    return 0;
  }

  if (param.set_stdin)
  {
    int res = setvbuf(stdin, NULL, _IONBF, 0);
    LOG_TRACE("setvbuf(stdin...) return %d", res);
  }
  if (param.set_stdout)
  {
    int res = setvbuf(stdout, NULL, _IONBF, 0);
    LOG_TRACE("setvbuf(stdout...) return %d", res);
  }
  if (param.set_stderr)
  {
    int res = setvbuf(stderr, NULL, _IONBF, 0);
    LOG_TRACE("setvbuf(stderr...) return %d", res);
  }

  for (int i = 0; i < param.count; i++)
  {
    QString s = QString::number(i);
    s += param.msg;
    if (param.linefeed) s += "\n";
    printf("%s", qPrintable(s));
    if (param.flush) fflush(stdout);
    LOG_TRACE("src %s", qPrintable(s));
    msleep(param.interval);
  }

  return 0;
}
